#include <LCD_I2C.h>
#include <Keypad.h>
#include <stdlib.h>

//Reset
void(* resetFunc) (void) = 0;

//Keypad
const byte ROWS = 4;
const byte COLS = 3;
char keys[ROWS][COLS] = 
{
  {'1', '2', '3'},
  {'4', '5', '6'},
  {'7', '8', '9'},
  {'*', '0', '#'}
};

byte rowPins[ROWS] = {7, 2, 3, 5};
byte colPins[COLS] = {6, 8, 4};

Keypad keyPad = Keypad(makeKeymap(keys), rowPins, colPins, ROWS, COLS);

const byte NUMBER = 10;
const byte TICK = 5;
//Morse Code
int morseIndex = 0;
unsigned long previousMillis3 = 0;
int MorseNumbers[NUMBER][TICK]
{
  {0, 0, 0, 0, 0}, // 0
  {1, 0, 0, 0, 0}, // 1
  {1, 1, 0, 0, 0}, // 2
  {1, 1, 1, 0, 0}, // 3
  {1, 1, 1, 1, 0}, // 4
  {1, 1, 1, 1, 1}, // 5
  {0, 1, 1, 1, 1}, // 6
  {0, 0, 1, 1, 1}, // 7
  {0, 0, 0, 1, 1}, // 8
  {0, 0, 0, 0, 1}, // 9
};

//Buzzer

int buzzerPin = 9;
unsigned long buzzerInterval = 50;
int state = LOW;
unsigned long previousMillis = 0;
unsigned long previousMillis2 = 0;
bool intervalMet = false;

//LCD
LCD_I2C lcd(0x27, 16, 2);
const long interval = 1000;

unsigned long startTime;
char buf[16];

int maxSecond = 60;
int secondSingle = 0;
int secondDouble = 0;

int minuteSingle = 0;
int minuteDouble = 0;

int hourSingle = 0;
int hourDouble = 0;

int second = 20;
int minute = 10;
int hour = 1;

//Led
int green = 10;
int red = A0;

//Bomba Variables
String ArmingCode = "";
int codeArray[8];

String DisarmingInput = "";
String TimerString = "";
bool isArmed = false;
bool isArming = false;
bool isDisarming = false;
bool isTimerSet = false;
bool isSettingTimer = false;

char ArmingKey = '*';
char TimerKey = '#';

int TotalTimerValue = 0;
int MaxTimerValue = 0;

int wireOne = 13;
int wireOneState = LOW;
bool oneTriggered = false;

int wireTwo = 12;
int wireTwoState = LOW;
bool twoTriggered = false;

int wireThree = 11;
int wireThreeState = LOW;
bool threeTriggered = false;


void setup() 
{
  Serial.begin(9600);
  
  lcd.begin();
  lcd.backlight();

  lcd.setCursor(4,0);
  lcd.print("BOMBA V1");

  pinMode(wireOne, INPUT_PULLUP);
  pinMode(wireTwo, INPUT_PULLUP);
  pinMode(wireThree, INPUT_PULLUP);

  pinMode(green, OUTPUT);
  pinMode(red, OUTPUT);

  digitalWrite(green, LOW);
  digitalWrite(red, LOW);
  
  pinMode(buzzerPin, OUTPUT);
  digitalWrite(buzzerPin, LOW);
}

void loop() 
{ 
  wireOneState = digitalRead(wireOne);
  wireTwoState = digitalRead(wireTwo);
  wireThreeState = digitalRead(wireThree);


  if(wireOneState == LOW && oneTriggered && wireThreeState == LOW && threeTriggered && twoTriggered)
  {
      resetFunc();
  }
  
  if(wireOneState != LOW && !oneTriggered)
  {
    oneTriggered = true;
    Serial.println(wireOneState);
    hour = 0;
    minute = 10;
    second = 0;
    SetTimerVariables();
    
  }
   if(wireTwoState != LOW && !twoTriggered)
  {
    twoTriggered = true;
    Serial.println(wireTwoState);
    hour = 0;
    minute = 0;
    second = 30;
    SetTimerVariables();
  }
   if(wireThreeState != LOW && !threeTriggered)
  {
    threeTriggered = true;
    Serial.println(wireThreeState);
    hour = 0;
    minute = 2;
    second = 45;
    SetTimerVariables();
  }
  
  DetectInput();
  if(isDisarming)
  {
    PlayMorse(DisarmingInput.length());
  }
  
  if(isArmed)
  {
   digitalWrite(green, LOW);
   CountDown();
  }

  if(!isArmed)
  {
    digitalWrite(green, HIGH);
    digitalWrite(buzzerPin, LOW);  
  }
}

void DetectInput()
{ 
  char key = keyPad.getKey();
  if(key)
  {
    if(!isArmed)
    {
       Blip(buzzerPin, 0, 40);
       
    }
    
    if(isArming)
    {
     Arming(key);
     return;
    }

    if(isDisarming)
    {
      Disarming(key);  
      return;
    }

    if(isSettingTimer)
    {
      SetTimer(key);
      return;
    }
    
    if(key == ArmingKey && !isArming && !isArmed)
    {
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("ARMING SEQUENCE");
      ArmingCode = "";
      isArming = true;
    }
    
    if(key == ArmingKey && isArmed)
    {
      lcd.clear();
      lcd.setCursor(0,1);
      lcd.print("DISARMING");
      DisarmingInput = "";
      isDisarming = true;

      Blip(buzzerPin, 0, 80);
      Blip(buzzerPin, 100, 100);
    }

    if(key == TimerKey && !isSettingTimer && !isArmed && !isArming)
    {
      isTimerSet = false;
      TimerString = "";
      isSettingTimer = true;
    }
  } 
}

void Arming(char _key)
{
  int index = ArmingCode.length();
   //Set Arming Code
    if(_key != ArmingKey && _key != TimerKey && ArmingCode.length() < 8)
    {
      if(index == 0)
      {
        lcd.clear();
        lcd.setCursor(0,0);
        lcd.print("ARMING CODE");
      }

      if(index > 0)
      {
        lcd.setCursor(8 + index - 1, 1);
        lcd.print("*");  
      }
      
      if(index <= 7)
      { 
        lcd.setCursor(8 + index, 1);
        ArmingCode += _key;
        lcd.print(_key);
      }
    }

    if(index == 8 && _key == ArmingKey)
    { 
      for(int i = 0; i < ArmingCode.length(); i++)
      {
        codeArray[i] = ArmingCode.charAt(i) - '0';
        
        Serial.println(codeArray[i]);  
      }
      
     
      lcd.clear();
      isArming = false;

      if(isTimerSet)
      {
        isArmed = true;
      }
      else
      {
        isSettingTimer = true;  
        isArmed = false;

        SetTimer(_key);
      }
    }   
}

void PlayMorse(int _index)
{ 
  //Timer
  unsigned long currentMillis = millis();
  if(currentMillis - previousMillis2 >= 1000)
  {
    Blip(buzzerPin, 0, 25); 
    int codeIndex = ArmingCode.charAt(_index) - '0';
    int tickIndex = morseIndex;
    if(morseIndex > 0)
    {
      tickIndex = morseIndex - 1;  
    }
    int tick = MorseNumbers[codeIndex][tickIndex];

    Serial.print("Index: ");
    Serial.print(tickIndex);
    Serial.print(" Number: ");
    Serial.print(codeIndex);
    Serial.print(" Tick: ");
    Serial.println(tick);
    
    if(tick == 1 && tickIndex <= 4)
    {
      Blip(green, 0, 50); 
    }
    else if(tickIndex > 4)
    {
      morseIndex = 0;
      Blip(red, 0, 100);
      Serial.println("Sequence END");
    }
    
    previousMillis2 = currentMillis;
    morseIndex++;
  }
}

void Disarming(char _key)
{
  int index = DisarmingInput.length();
  if(index <= 0)
  { 
    for(int i = 0; i < 16; i++)
    {
      lcd.setCursor(0 + i, 1);
      lcd.print(" ");  
    }
    
    for(int i = 0; i < 8; i++)
    {
      lcd.setCursor(8 + i, 1);
      lcd.print("*");  
    } 
  }
  
  if(_key != ArmingKey && _key != TimerKey)
  {
    if(index < 8)
    { 
      lcd.setCursor(8 + DisarmingInput.length(), 1);
      DisarmingInput += _key;
      lcd.print(_key);
      morseIndex = 6;
    }

    if(index == 7)
    {
      if(DisarmingInput == ArmingCode)
      {
        lcd.setCursor(0, 1);
        lcd.print("DEFUSED");

        Blip(buzzerPin, 0, 100);
        Blip(buzzerPin, 150, 50);
        Blip(buzzerPin, 300, 80);
        
        isTimerSet = false;
        isArmed = false; 
        isDisarming = false;  
        TimerString = "";
        ArmingCode = "";
      }
      else
      {
        DisarmingInput = "";
        lcd.setCursor(0, 1);
        lcd.print("WRONG");  
      }
    }
  }
}

void SetTimer(char _key)
{
  int index = TimerString.length();
  
  if(index == 0)
  {
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("SETTING TIMER");
    lcd.setCursor(0, 1);
    lcd.print("00:00:00");  
  }

  if(_key != ArmingKey && _key != TimerKey)
  { 
    if(index < 6)
    {
      if(index < 2)
      {
        lcd.setCursor(index, 1);  
      }
      else if(index < 4)
      {
        if(index == 2 && _key - '0' > 5)
        {
          return;
        }
        lcd.setCursor(1 + index, 1);  
      }
      else 
      {
        if(index == 4 && _key - '0' > 5)
        {
          return;
        }
        lcd.setCursor(2 + index, 1);  
      }
      TimerString += _key;
      lcd.print(_key);
    }
  }

  if(index == 6 && _key == TimerKey)
  {
    hour = TimerString.substring(0, 2).toInt();
    minute = TimerString.substring(2, 4).toInt();
    second = TimerString.substring(4, 6).toInt(); 

    isSettingTimer = false;
    isTimerSet = true;
    SetTimerVariables();

    Serial.println(TimerString);

    lcd.setCursor(0, 0);
    lcd.print("TIMER SET     ");
    if(ArmingCode.length() == 8)
    {
      lcd.clear();
      isArmed = true;  
    }
  } 
}

void SetTimerVariables()
{
  TotalTimerValue = abs((hour * 3600) + (minute * 60) + second);
  MaxTimerValue = TotalTimerValue;
  
  Serial.println(TotalTimerValue);
  if(second < 10)
  {
    secondSingle = second;
    secondDouble = 0;
  }
  else
  {
    secondSingle = second % 10;
    secondDouble = second / 10;
  }

  if(minute < 10)
  {
    minuteSingle = minute;
    minuteDouble = 0;
  }
  else
  {
    minuteSingle = minute % 10;
    minuteDouble = minute / 10;  
  }

  if(hour < 10)
  {
    hourSingle = hour;
    hourDouble = 0;  
  }
  else
  {
    hourSingle = hour % 10;
    hourDouble = hour / 10;  
  }  
}

void Blip(int pin, unsigned long _interval, unsigned long _duration)
{
  unsigned long currentMillis = millis();
  if(currentMillis - previousMillis2 >= _interval)
  {
    previousMillis2 = currentMillis;
    state = HIGH;
    digitalWrite(pin, state);
    delay(_duration);
    state = LOW;
    digitalWrite(pin, state);
    delay(_duration);
  }
}

void CountDown()
{
  unsigned long currentMillis = millis();
  unsigned long timer = currentMillis - previousMillis;
  unsigned long timerInterval = map(TotalTimerValue, 0, MaxTimerValue, 5, 3000);
  if(!isDisarming)
  {
    Blip(red, timerInterval, 20);
    Blip(buzzerPin, timerInterval, 20);
  }
  
  if(timer > interval)
  {
    previousMillis = currentMillis;
    
    if(TotalTimerValue <= 0 && isArmed)
    {
      //BABOOM!!!
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("BOOOOOOOM");
      isArmed = false;
      return;
    }
    
    secondSingle -= 1;
    TotalTimerValue -= 1;

    if(secondSingle < 0 && secondDouble > 0)
    {
        secondDouble -= 1;
        secondSingle = 9;
    }
    
    if(secondDouble >= 0 && secondSingle < 0)
    {
      secondDouble = 5;
      secondSingle = 9;
      
      if(minuteDouble > 0 && minuteSingle <= 0)
      {
        minuteDouble -= 1;
        minuteSingle = 9;
      }
      else
      {
        minuteSingle -= 1;    
      }
    }
 
    if(minuteDouble >= 0 && minuteSingle < 0)
    {
      minuteDouble = 5;
      minuteSingle = 9;
      
      if(hourDouble > 0 && hourSingle <= 0)
      {
        hourDouble -= 1;
        hourSingle = 9;  
      }
      else
      {
        hourSingle -= 1;
      }
    }

    if(hourSingle < 0 && hourDouble < 0)
    {
      hourSingle = 0;
      hourDouble = 0;
    }

    PrintCountDown();
  }
}

void PrintCountDown()
{
 lcd.setCursor(15, 0);
  lcd.print(secondSingle);

  lcd.setCursor(14, 0);
  lcd.print(secondDouble);

  lcd.setCursor(13, 0);
  lcd.print(":");

  lcd.setCursor(12, 0);
  lcd.print(minuteSingle);
  
  lcd.setCursor(11, 0);
  lcd.print(minuteDouble);

  lcd.setCursor(10, 0);
  lcd.print(":");

  lcd.setCursor(9, 0);
  lcd.print(hourSingle);

  lcd.setCursor(8, 0);
  lcd.print(hourDouble);
}
